#!/usr/local/bin/perl -w

=head1 NAME

identity.t - test VCP::Filter::identity

=cut

use strict ;

use Carp ;
use File::Spec ;
use Test ;
use VCP::TestUtils ;

## These next few are for in vitro testing
use VCP::Debug qw( :debug );
use VCP::Filter::map;
use VCP::Rev;

enable_debug( split /,/, $ENV{VCPDEBUG} ) if defined $ENV{VCPDEBUG} ;



my %seen ;
my @perl = ( $^X, map {
      my $s = $_ ;
      $s = File::Spec->rel2abs( $_ ) ;
      "-I$s" ;
   } grep ! $seen{$_}++, @INC
) ;

## We always run vcp by doing a @perl, vcp, to make sure that vcp runs under
## the same version of perl that we are running under.
my $vcp = 'vcp' ;
$vcp = "bin/$vcp"    if -e "bin/$vcp" ;
$vcp = "../bin/$vcp" if -e "../bin/$vcp" ;

$vcp = File::Spec->rel2abs( $vcp ) ;

my @vcp = ( @perl, $vcp ) ;


my $t = -d 't' ? 't/' : '' ;

sub r {
   my ( $name, $branch_id ) = $_[0] =~ /\A(?:(.+?))?(?:<(.*)>)?\z/
      or die "Couldn't parse '$_[0]'";

   VCP::Rev->new(
      id        => $_[0],
      name      => $name,
      branch_id => $branch_id,
   )
}

my $sub;

my $r_out;

# HACK
sub VCP::Filter::map::dest {
    return "main";
}

sub handle_rev {
    my $self = shift;
    my ( $rev ) = @_;
    $r_out = join "", $rev->name || "", "<", $rev->branch_id || "", ">";
}

sub t {
    return skip "compilation failed", 1 unless $sub;

    my ( $expr, $expected ) = @_;

    $r_out = undef;

    $sub->( r $expr );

    @_ = ( $r_out || "<<deleted>>", $expected || "<<deleted>>" );
    goto &ok;
}

my @tests = (
## In vitro tests
sub {
   $sub = eval { VCP::Filter::map->_compile_rules(
      [ '<b>',          '<B>'          ],
      [ 'a',            'A'            ],
      [ 'a',            'NONONO'       ],
      [ 'c<d>',         'C<D>'         ],
      [ 'xyz',          '<<keep>>'     ],
      [ 'x*',           '<<delete>>'   ],
      [ 's(*)v<(...)>', 'S$1V${2}Y<>'  ],
      [ 's(*)v<>',      'NONONO'       ],
   ) }; 
   ok defined $sub || $@, 1;
},

sub { t "a<b>",     "a<B>"       },
sub { t "a<c>",     "A<c>"       },
sub { t "c<d>",     "C<D>"       },
sub { t "c<e>",     "c<e>"       },
sub { t "e<d>",     "e<d>"       },
sub { t "xab",      undef        },
sub { t "xyz",      "xyz<>"      },
sub { t "Z<Z>",     "Z<Z>"       },
sub { t "stuv<wx>", "StuVwxY<>"  },

## In vivo tests
sub {
  eval {
     my $out ;
     my $infile = "t/test-revml-in-0-no-big-files.revml";
     ## $in and $out allow us to avoide execing diff most of the time.
     run [ @vcp, "vcp:-" ], \<<'END_VCP', \$out;
Source: t/test-revml-in-0-no-big-files.revml

Destination: - --sort-by=name,rev_id

Map:
END_VCP

     my $in = slurp $infile;
     assert_eq $infile, $in, $out ;
  } ;
  ok $@ || '', '', 'diff' ;
},

);@tests=(
sub {
  eval {
     my $out ;
     my $infile = "t/test-revml-in-0-no-big-files.revml";
     ## $in and $out allow us to avoide execing diff most of the time.
     run [ @vcp, "vcp:-" ], \<<'END_VCP', \$out;
Source: t/test-revml-in-0-no-big-files.revml

Destination: - --sort-by=name,rev_id

Map:
    add/f(*)   hey/a${1}b
    add/f(1)   hey/a$1b
    add/f(2)   hey/a${1}b
END_VCP
     my $in = slurp $infile;

     $in =~ s{(name>)add/f([^<]*)}{$1hey/a$2b}g;

     assert_eq $infile, $in, $out ;
  } ;
  ok $@ || '', '', 'diff' ;
},

) ;

plan tests => scalar( @tests ) ;

unless ( -e $vcp ) {
   print STDERR "# '$vcp' not found\n" ;
   skip( 1, '' ) for @tests ;
   exit ;
}

$_->() for @tests ;
